@@ -13,6 +13,11 @@ from .terminator import Terminator
 from .config import Config
 from .factory import Factory
 from .util import dbg, err, enumerate_descendants
+from .terminal import Terminal
+from .container import Container
+from .configjson import ConfigJson
+from gi.repository import Gtk as gtk
+from gi.repository import GObject as gobject
 
 CONFIG = Config()
 if not CONFIG['dbus']:
@@ -52,14 +57,14 @@ class DBusService(Borg, dbus.service.Object):
             except Exception as e:
                 err('Unable to connect to DBUS Server, proceeding as standalone')
                 raise ImportError
-            proxy = bus.get_object('org.freedesktop.DBus', 
+            proxy = bus.get_object('org.freedesktop.DBus',
                                    '/org/freedesktop/DBus')
             flags = 1 | 4 # allow replacement | do not queue
             if not proxy.RequestName(BUS_NAME, dbus.UInt32(flags)) in (1, 4):
                 dbg('bus name unavailable: %s' % BUS_NAME)
                 raise dbus.exceptions.DBusException(
                     "Couldn't get DBus name %s: Name exists" % BUS_NAME)
-            self.bus_name = dbus.service.BusName(BUS_NAME, 
+            self.bus_name = dbus.service.BusName(BUS_NAME,
                                                  bus=dbus.SessionBus())
         if not self.bus_path:
             self.bus_path = BUS_PATH
@@ -70,6 +75,15 @@ class DBusService(Borg, dbus.service.Object):
     def new_window_cmdline(self, options=dbus.Dictionary()):
         """Create a new Window"""
         dbg('dbus method called: new_window with parameters %s'%(options))
+        if options['configjson']:
+            dbg(options['configjson'])
+            configjson = ConfigJson()
+            layoutname = configjson.extend_config(options['configjson'])
+            if layoutname and ((not options['layout']) or options['layout'] == 'default'):
+                options['layout'] = layoutname
+                if not options['profile']:
+                    options['profile'] = configjson.get_profile_to_use()
+
         oldopts = self.terminator.config.options_get()
         oldopts.__dict__ = options
         self.terminator.config.options_set(oldopts)
@@ -110,18 +124,84 @@ class DBusService(Borg, dbus.service.Object):
         """Create a new tab"""
         return self.new_terminal(uuid, 'tab')
 
-    @dbus.service.method(BUS_NAME)
-    def hsplit(self, uuid=None):
-        """Split a terminal horizontally, by UUID"""
-        return self.new_terminal(uuid, 'hsplit')
+    @dbus.service.method(BUS_NAME) 
+    def bg_img_all (self,options=dbus.Dictionary()):
+        for terminal in self.terminator.terminals:
+            terminal.set_background_image(options.get('file')) 
+            
+    @dbus.service.method(BUS_NAME) 
+    def bg_img(self,uuid=None,options=dbus.Dictionary()):
+        self.terminator.find_terminal_by_uuid(uuid).set_background_image(options.get('file'))
 
     @dbus.service.method(BUS_NAME)
-    def vsplit(self, uuid=None):
+    def hsplit(self, uuid=None,options=None):
+        """Split a terminal horizontally, by UUID"""
+        if options:
+            cmd = options.get('execute')
+            title = options.get('title')
+            return self.new_terminal_cmd(uuid=uuid, title=title, cmd=cmd, split_vert=True) 
+        else:
+            return self.new_terminal(uuid, 'hsplit')
+
+    @dbus.service.method(BUS_NAME)
+    def vsplit(self, uuid=None,options=None):
         """Split a terminal vertically, by UUID"""
-        return self.new_terminal(uuid, 'vsplit')
+        if options:
+            cmd = options.get('execute')
+            title = options.get('title')
+            return self.new_terminal_cmd(uuid=uuid, title=title, cmd=cmd, split_vert=False) 
+        else:
+            return self.new_terminal(uuid, 'vsplit')
+
+    def get_terminal_container(self, terminal, container=None):
+        terminator = Terminator()
+        if not container:
+            for window in terminator.windows:
+                owner = self.get_terminal_container(terminal, window)
+                if owner: return owner
+        else:
+            for child in container.get_children():
+                if isinstance(child, Terminal) and child == terminal:
+                    return container
+                if isinstance(child, Container):
+                    owner = self.get_terminal_container(terminal, child)
+                    if owner: return owner
+
+    def new_terminal_cmd(self, uuid=None, title=None, cmd=None, split_vert=False):
+        """Split a terminal by UUID and immediately runs the specified command in the new terminal"""
+        if not uuid:
+            return "ERROR: No UUID specified"
+
+        terminal = self.terminator.find_terminal_by_uuid(uuid)
+
+        terminals_before = set(self.get_terminals())
+        if not terminal:
+            return "ERROR: Terminal with supplied UUID not found"
+
+        # get current working dir out of target terminal
+        cwd = terminal.get_cwd()
+
+        # get current container
+        container = self.get_terminal_container(terminal)
+        maker = Factory()
+        sibling = maker.make('Terminal')
+        sibling.set_cwd(cwd)
+        if title: sibling.titlebar.set_custom_string(title)
+        sibling.spawn_child(init_command=cmd)
+
+        # split and run command in new terminal
+        container.split_axis(terminal, split_vert, cwd, sibling)
+
+        terminals_after = set(self.get_terminals())
+        # Detect the new terminal UUID
+        new_terminal_set = list(terminals_after - terminals_before)
+        if len(new_terminal_set) != 1:
+            return "ERROR: Cannot determine the UUID of the added terminal"
+        else:
+            return new_terminal_set[0]
 
     def new_terminal(self, uuid, type):
-        """Split a terminal horizontally or vertically, by UUID"""
+        """Split a terminal horizontally o?r vertically, by UUID"""
         dbg('dbus method called: %s' % type)
         if not uuid:
             return "ERROR: No UUID specified"
@@ -150,6 +230,13 @@ class DBusService(Borg, dbus.service.Object):
         """Return a list of all the terminals"""
         return [x.uuid.urn for x in self.terminator.terminals]
 
+    @dbus.service.method(BUS_NAME)
+    def get_focused_terminal(self):
+        """Returns the uuid of the currently focused terminal"""
+        if self.terminator.last_focused_term:
+            return self.terminator.last_focused_term.uuid.urn
+        return None
+
     @dbus.service.method(BUS_NAME)
     def get_window(self, uuid=None):
         """Return the UUID of the parent window of a given terminal"""
@@ -197,6 +284,24 @@ class DBusService(Borg, dbus.service.Object):
                 if terminal in terms:
                     return root_widget.get_tab_label(tab_child).get_label()
 
+    @dbus.service.method(BUS_NAME)
+    def set_tab_title(self, uuid=None, options=dbus.Dictionary()):
+        """Set the title of a parent tab of a given terminal"""
+        tab_title = options.get('tab-title')
+
+        maker = Factory()
+        terminal = self.terminator.find_terminal_by_uuid(uuid)
+        window = terminal.get_toplevel()
+
+        if not window.is_child_notebook():
+            return
+
+        notebook = window.get_children()[0]
+        n_page = notebook.get_current_page()
+        page = notebook.get_nth_page(n_page)
+        label = notebook.get_tab_label(page)
+        label.set_custom_label(tab_title, force=True)
+
     @dbus.service.method(BUS_NAME)
     def switch_profile(self, uuid=None, options=dbus.Dictionary()):
         """Switch profile of a given terminal"""
@@ -204,6 +309,14 @@ class DBusService(Borg, dbus.service.Object):
         profile_name = options.get('profile')
         terminal.force_set_profile(False, profile_name)
 
+    @dbus.service.method(BUS_NAME)
+    def switch_profile_all(self, options=dbus.Dictionary()):
+        """Switch profile of a given terminal"""
+        for terminal in self.terminator.terminals:
+            profile_name = options.get('profile')
+            terminal.force_set_profile(False, profile_name)
+
+
 def with_proxy(func):
     """Decorator function to connect to the session dbus bus"""
     dbg('dbus client call: %s' % func.__name__)
@@ -217,7 +330,7 @@ def with_proxy(func):
                 "Remotinator can't connect to terminator. " +
                 "May be terminator is not running.")
 
-        func(proxy, *args, **argd)
+        return func(proxy, *args, **argd)
     return _exec
 
 @with_proxy
@@ -247,18 +360,23 @@ def new_tab(session, uuid, options):
 @with_proxy
 def hsplit(session, uuid, options):
     """Call the dbus method to horizontally split a terminal"""
-    print(session.hsplit(uuid))
+    print(session.hsplit(uuid,options))
 
 @with_proxy
 def vsplit(session, uuid, options):
     """Call the dbus method to vertically split a terminal"""
-    print(session.vsplit(uuid))
+    print(session.vsplit(uuid,options))
 
 @with_proxy
 def get_terminals(session, options):
     """Call the dbus method to return a list of all terminals"""
     print('\n'.join(session.get_terminals()))
 
+@with_proxy
+def get_focused_terminal(session, options):
+    """Call the dbus method to return the currently focused terminal"""
+    return session.get_focused_terminal()
+
 @with_proxy
 def get_window(session, uuid, options):
     """Call the dbus method to return the toplevel tab for a terminal"""
@@ -279,8 +397,25 @@ def get_tab_title(session, uuid, options):
     """Call the dbus method to return the title of a tab"""
     print(session.get_tab_title(uuid))
 
+@with_proxy
+def set_tab_title(session, uuid, options):
+    """Call the dbus method to set the title of a tab"""
+    session.set_tab_title(uuid, options)
+
 @with_proxy
 def switch_profile(session, uuid, options):
     """Call the dbus method to return the title of a tab"""
     session.switch_profile(uuid, options)
 
+@with_proxy
+def switch_profile_all(session,options):
+    """Call the dbus method to return the title of a tab"""
+    session.switch_profile_all(options)
+
+@with_proxy
+def bg_img_all(session,options):
+    session.bg_img_all(options)
+
+@with_proxy
+def bg_img(session,uuid,options):
+    session.bg_img(uuid,options)
