@@ -118,6 +118,7 @@ class PrefsEditor:
                         'go_right'         : _('Focus the terminal right'),
                         'rotate_cw'        : _('Rotate terminals clockwise'),
                         'rotate_ccw'       : _('Rotate terminals counter-clockwise'),
+                        'split_auto'       : _('Split automatically'),
                         'split_horiz'      : _('Split horizontally'),
                         'split_vert'       : _('Split vertically'),
                         'close_term'       : _('Close terminal'),
@@ -173,18 +174,19 @@ class PrefsEditor:
                         'broadcast_group'  : _('Broadcast key presses to group'),
                         'broadcast_all'    : _('Broadcast key events to all'),
                         'insert_number'    : _('Insert terminal number'),
-                        'insert_padded'    : _('Insert padded terminal number'),
+                        'insert_padded'    : _('Insert zero padded terminal number'),
                         'edit_window_title': _('Edit window title'),
                         'edit_terminal_title': _('Edit terminal title'),
                         'edit_tab_title'   : _('Edit tab title'),
                         'layout_launcher'  : _('Open layout launcher window'),
                         'next_profile'     : _('Switch to next profile'),
                         'previous_profile' : _('Switch to previous profile'), 
-			'preferences'	   : _('Open the Preferences window'),
+                        'preferences'	   : _('Open the Preferences window'),
+                        'preferences_keybindings' : _('Open the Preferences-Keybindings window'),
                         'help'             : _('Open the manual')
             }
 
-    def __init__ (self, term):
+    def __init__ (self, term, cur_page=0):
         self.config = config.Config()
         self.config.base.reload()
         self.term = term
@@ -227,6 +229,10 @@ class PrefsEditor:
             err('Unable to set values: %s' % e)
         self.config.uninhibit_save()
 
+        guiget = self.builder.get_object
+        nb = guiget('notebook1')
+        nb.set_current_page(cur_page)
+
     def on_closebutton_clicked(self, _button):
         """Close the window"""
         terminator = Terminator()
@@ -415,7 +421,47 @@ class PrefsEditor:
         selection.connect('changed', self.on_layout_item_selection_changed)
 
         ## Keybindings tab
-        widget = guiget('keybindingtreeview')
+        widget   = guiget('keybindingtreeview')
+        kbsearch = guiget('keybindingsearchentry')
+        self.keybind_filter_str = ""
+
+        #lets hide whatever we can in nested scope
+        def filter_visible(model, treeiter, data):
+            act  = model[treeiter][0]
+            keys = data[act] if act in data else ""
+            desc = model[treeiter][1]
+            kval = model[treeiter][2]
+            mask = model[treeiter][3]
+            #so user can search for disabled keys also
+            if not (len(keys) and kval and mask):
+                act = "Disabled"
+
+            self.keybind_filter_str = self.keybind_filter_str.lower()
+            searchtxt = (act + " " + keys + " " + desc).lower()
+            pos = searchtxt.find(self.keybind_filter_str)
+            if (pos >= 0):
+                dbg("filter find:%s in search text: %s" %
+                                (self.keybind_filter_str, searchtxt))
+                return True
+
+            return False
+
+        def on_search(widget, text):
+            MAX_SEARCH_LEN = 10
+            self.keybind_filter_str = widget.get_text()
+            ln = len(self.keybind_filter_str)
+            #its a small list & we are eager for quick search, but limit
+            if (ln >=2 and ln < MAX_SEARCH_LEN):
+                dbg("filter search str: %s" % self.keybind_filter_str)
+                self.treemodelfilter.refilter()
+
+        def on_search_refilter(widget):
+            dbg("refilter")
+            self.treemodelfilter.refilter()
+
+        kbsearch.connect('key-press-event', on_search)
+        kbsearch.connect('backspace', on_search_refilter)
+
         liststore = widget.get_model()
         liststore.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         keybindings = self.config['keybindings']
@@ -431,6 +477,10 @@ class PrefsEditor:
             liststore.append([keybinding, self.keybindingnames[keybinding],
                              keyval, mask])
 
+        self.treemodelfilter = liststore.filter_new()
+        self.treemodelfilter.set_visible_func(filter_visible, keybindings)
+        widget.set_model(self.treemodelfilter)
+
         ## Plugins tab
         # Populate the plugin list
         widget = guiget('pluginlist')
@@ -624,6 +674,10 @@ class PrefsEditor:
         widget.set_value(float(self.config['inactive_color_offset']))
         widget = guiget('inactive_color_offset_value_label')
         widget.set_text('%d%%' % (int(float(self.config['inactive_color_offset'])*100)))
+        widget = guiget('inactive_bg_color_offset')
+        widget.set_value(float(self.config['inactive_bg_color_offset']))
+        widget = guiget('inactive_bg_color_offset_value_label')
+        widget.set_text('%d%%' % (int(float(self.config['inactive_bg_color_offset'])*100)))
         # Open links with a single click (instead of a Ctrl-left click)
         widget = guiget('link_single_click')
         widget.set_active(self.config['link_single_click'])
@@ -644,11 +698,42 @@ class PrefsEditor:
         elif self.config['background_type'] == 'image':
             guiget('image_radiobutton').set_active(True)
         self.update_background_tab()
+        # Background image
+        widget = guiget('background_image_file')
+        widget.set_filename(self.config['background_image'])
+
+        widget = guiget('background_image_mode_combobox')
+        if self.config['background_image_mode'] == 'scale_and_fit':
+            widget.set_active(1)
+        elif self.config['background_image_mode'] == 'scale_and_crop':
+            widget.set_active(2)
+        elif self.config['background_image_mode'] == 'tiling':
+            widget.set_active(3)
+        else:
+            # default to stretch_and_fill
+            widget.set_active(0)
+
+        widget = guiget('background_image_align_horiz_combobox')
+        if self.config['background_image_align_horiz'] == 'center':
+            widget.set_active(1)
+        elif self.config['background_image_align_horiz'] == 'right':
+            widget.set_active(2)
+        else:
+            # default to left
+            widget.set_active(0)
+
+        widget = guiget('background_image_align_vert_combobox')
+        if self.config['background_image_align_vert'] == 'middle':
+            widget.set_active(1)
+        elif self.config['background_image_align_vert'] == 'bottom':
+            widget.set_active(2)
+        else:
+            # default to top
+            widget.set_active(0)
+
         # Background shading
         widget = guiget('background_darkness_scale')
         widget.set_value(float(self.config['background_darkness']))
-        widget = guiget('background_image_file')
-        widget.set_filename(self.config['background_image'])
    
         ## Scrolling tab
         # Scrollbar position
@@ -666,7 +751,7 @@ class PrefsEditor:
         # Scrollback infinite
         widget = guiget('scrollback_infinite')
         widget.set_active(self.config['scrollback_infinite'])
-        # Scroll on outut
+        # Scroll on output
         widget = guiget('scroll_on_output_checkbutton')
         widget.set_active(self.config['scroll_on_output'])
         # Scroll on keystroke
@@ -949,6 +1034,41 @@ class PrefsEditor:
         self.config['background_image'] = widget.get_filename()
         self.config.save()
 
+    def on_background_image_mode_changed(self, widget):
+        selected = widget.get_active()
+        if selected == 1:
+            value = 'scale_and_fit'
+        elif selected == 2:
+            value = 'scale_and_crop'
+        elif selected == 3:
+            value = 'tiling'
+        else:
+            value = 'stretch_and_fill'
+        self.config['background_image_mode'] = value
+        self.config.save()
+
+    def on_background_image_align_horiz_changed(self, widget):
+        selected = widget.get_active()
+        if selected == 1:
+            value = 'center'
+        elif selected == 2:
+            value = 'right'
+        else:
+            value = 'left'
+        self.config['background_image_align_horiz'] = value
+        self.config.save()
+
+    def on_background_image_align_vert_changed(self, widget):
+        selected = widget.get_active()
+        if selected == 1:
+            value = 'middle'
+        elif selected == 2:
+            value = 'bottom'
+        else:
+            value = 'top'
+        self.config['background_image_align_vert'] = value
+        self.config.save()
+
     def on_darken_background_scale_value_changed(self, widget):
         """Background darkness setting changed"""
         value = widget.get_value()  # This one is rounded according to the UI.
@@ -1219,7 +1339,7 @@ class PrefsEditor:
         self.config.save()
 
     def on_title_transmit_bg_color_color_set(self, widget):
-        """Title transmit backgruond colour changed"""
+        """Title transmit background colour changed"""
         self.config['title_transmit_bg_color'] = color2hex(widget)
         self.config.save()
 
@@ -1244,6 +1364,17 @@ class PrefsEditor:
         label_widget = guiget('inactive_color_offset_value_label')
         label_widget.set_text('%d%%' % (int(value * 100)))
 
+    def on_inactive_bg_color_offset_value_changed(self, widget):
+        """Inactive background color offset setting changed"""
+        value = widget.get_value()  # This one is rounded according to the UI.
+        if value > 1.0:
+          value = 1.0
+        self.config['inactive_bg_color_offset'] = value
+        self.config.save()
+        guiget = self.builder.get_object
+        label_widget = guiget('inactive_bg_color_offset_value_label')
+        label_widget.set_text('%d%%' % (int(value * 100)))
+
     def on_handlesize_value_changed(self, widget):
         """Handle size changed"""
         value = widget.get_value()  # This one is rounded according to the UI.
@@ -1553,10 +1684,12 @@ class PrefsEditor:
         self.config['background_type'] = backtype
         self.config.save()
 
-        if backtype == 'image':
-                guiget('background_image_file').set_sensitive(True)
-        else:
-                guiget('background_image_file').set_sensitive(False)
+        # toggle sensitivity of widgets related to background image
+        for element in ('background_image_file',
+                        'background_image_align_horiz_combobox',
+                        'background_image_align_vert_combobox',
+                        'background_image_mode_combobox'):
+            guiget(element).set_sensitive(backtype == 'image')
 
         if backtype in ('transparent', 'image'):
             guiget('darken_background_scale').set_sensitive(True)
@@ -1730,6 +1863,11 @@ class PrefsEditor:
         self.config.save()
 
     def on_cellrenderer_accel_edited(self, liststore, path, key, mods, _code):
+        inpath = path #save for debugging
+        trpath = Gtk.TreePath.new_from_string(inpath)
+        path   = str(self.treemodelfilter.convert_path_to_child_path(trpath))
+        dbg("convert path with filter from: %s to: %s" %
+                                        (inpath, path))
         """Handle an edited keybinding"""
         # Ignore `Gdk.KEY_Tab` so that `Shift+Tab` is displayed as `Shift+Tab`
         # in `Preferences>Keybindings` and NOT `Left Tab` (see `Gdk.KEY_ISO_Left_Tab`).
@@ -1800,6 +1938,12 @@ class PrefsEditor:
         self.config.save()
 
     def on_cellrenderer_accel_cleared(self, liststore, path):
+        inpath = path #save for debugging
+        trpath = Gtk.TreePath.new_from_string(inpath)
+        path   = str(self.treemodelfilter.convert_path_to_child_path(trpath))
+        dbg("convert path with filter from: %s to: %s" %
+                                        (inpath, path))
+
         """Handle the clearing of a keybinding accelerator"""
         celliter = liststore.get_iter_from_string(path)
         liststore.set(celliter, 2, 0, 3, 0)
