@@ -6,8 +6,7 @@
 import os
 import signal
 import gi
-import cairo
-from gi.repository import GLib, GObject, Pango, Gtk, Gdk, GdkPixbuf
+from gi.repository import GLib, GObject, Pango, Gtk, Gdk, GdkPixbuf, cairo
 gi.require_version('Vte', '2.91')  # vte-0.38 (gnome-3.14)
 from gi.repository import Vte
 import subprocess
@@ -32,46 +31,24 @@ from . import plugin
 from terminatorlib.layoutlauncher import LayoutLauncher
 from . import regex
 
-class Overpaint(Vte.Terminal):
-    def __init__(self):
-        Vte.Terminal.__init__(self)
-        self.config = Config()
-        ### inactive_color_offset is the opposite of alpha level
-        self.dim_p = float(self.config['inactive_color_offset'])
-        self.dim_l = round(1.0 - self.dim_p,3)
-    def dim(self,b):
-        self.overpaint = b
-
-    def do_draw(self,cr):
-        ### get_color_background_for_draw is not available in older 
-        ### versions of vte
-        try:
-            bgc = Vte.Terminal.get_color_background_for_draw(self)
-        except AttributeError as e:
-            bgc = Gdk.RGBA()
-            bgc.parse(self.config['background_color'])
-        Vte.Terminal.do_draw(self,cr)
-        if self.overpaint:
-            bgc.alpha = self.dim_l
-            cr.set_operator(cairo.Operator.OVER)
-            Gdk.cairo_set_source_rgba(cr,bgc)
-            cr.rectangle(0.0,0.0,self.get_allocated_width(),self.get_allocated_height())
-            cr.paint()
-
 # pylint: disable-msg=R0904
 class Terminal(Gtk.VBox):
     """Class implementing the VTE widget and its wrappings"""
 
     __gsignals__ = {
+        'pre-close-term': (GObject.SignalFlags.RUN_LAST, None, ()),
         'close-term': (GObject.SignalFlags.RUN_LAST, None, ()),
         'title-change': (GObject.SignalFlags.RUN_LAST, None,
             (GObject.TYPE_STRING,)),
+        'insert-term-name': (GObject.SignalFlags.RUN_LAST, None, ()),
         'enumerate': (GObject.SignalFlags.RUN_LAST, None,
             (GObject.TYPE_INT,)),
         'group-tab': (GObject.SignalFlags.RUN_LAST, None, ()),
         'group-tab-toggle': (GObject.SignalFlags.RUN_LAST, None, ()),
         'ungroup-tab': (GObject.SignalFlags.RUN_LAST, None, ()),
         'ungroup-all': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'split-auto': (GObject.SignalFlags.RUN_LAST, None,
+            (GObject.TYPE_STRING,)),
         'split-horiz': (GObject.SignalFlags.RUN_LAST, None,
             (GObject.TYPE_STRING,)),
         'split-vert': (GObject.SignalFlags.RUN_LAST, None,
@@ -96,6 +73,9 @@ class Terminal(Gtk.VBox):
         'group-all-toggle': (GObject.SignalFlags.RUN_LAST, None, ()),
         'move-tab': (GObject.SignalFlags.RUN_LAST, None,
             (GObject.TYPE_STRING,)),
+        'group-win': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'group-win-toggle': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'ungroup-win': (GObject.SignalFlags.RUN_LAST, None, ()),
     }
 
     TARGET_TYPE_VTE = 8
@@ -122,8 +102,6 @@ class Terminal(Gtk.VBox):
     matches = None
     regex_flags = None
     config = None
-    default_encoding = None
-    custom_encoding = None
     custom_font_size = None
     layout_command = None
     relaunch_command = None
@@ -132,8 +110,11 @@ class Terminal(Gtk.VBox):
     is_held_open = False
 
     fgcolor_active = None
+    fgcolor_inactive = None
     bgcolor = None
+    bgcolor_inactive = None
     palette_active = None
+    palette_inactive = None
 
     composite_support = None
 
@@ -149,6 +130,7 @@ class Terminal(Gtk.VBox):
 
         # FIXME: Surely these should happen in Terminator::register_terminal()?
         self.connect('enumerate', self.terminator.do_enumerate)
+        self.connect('insert-term-name', self.terminator.do_insert_term_name)
         self.connect('focus-in', self.terminator.focus_changed)
         self.connect('focus-out', self.terminator.focus_left)
 
@@ -163,21 +145,7 @@ class Terminal(Gtk.VBox):
 
         self.pending_on_vte_size_allocate = False
 
-        self.vte = Overpaint()
-        self.vte.dim(False)
-        self.queue_draw()
-        self.background_image = None
-        if self.config['background_image'] != '':
-            try: 
-                self.background_image = GdkPixbuf.Pixbuf.new_from_file(self.config['background_image'])
-                self.vte.set_clear_background(False)
-                self.vte.connect("draw",self.background_draw)
-            except Exception as e:
-                self.background_image = None
-                self.vte.set_clear_background(True)
-                err('error loading background image: %s' % e)
-
-        self.background_alpha = self.config['background_darkness']
+        self.vte = Vte.Terminal()
         self.vte.set_allow_hyperlink(True)
         self.vte._draw_data = None
         if not hasattr(self.vte, "set_opacity") or \
@@ -187,9 +155,7 @@ class Terminal(Gtk.VBox):
             self.composite_support = True
         dbg('composite_support: %s' % self.composite_support)
 
-        
         self.vte.show()
-        self.default_encoding = self.vte.get_encoding()
         self.update_url_matches()
 
         self.terminalbox = self.create_terminalbox()
@@ -199,6 +165,7 @@ class Terminal(Gtk.VBox):
         self.titlebar.connect('edit-done', self.on_edit_done)
         self.connect('title-change', self.titlebar.set_terminal_title)
         self.titlebar.connect('create-group', self.really_create_group)
+        self.titlebar.update('window-focus-out')
         self.titlebar.show_all()
 
         self.searchbar = Searchbar()
@@ -226,6 +193,17 @@ class Terminal(Gtk.VBox):
         self.reconfigure()
         self.vte.set_size(80, 24)
 
+    def set_background_image(self,image):
+        try: 
+            bg_pixbuf = GdkPixbuf.Pixbuf.new_from_file(image)
+            self.background_image = Gdk.cairo_surface_create_from_pixbuf(bg_pixbuf, 1, None)
+            self.vte.set_clear_background(False)
+            self.vte.connect("draw", self.background_draw)
+        except Exception as e:
+            self.background_image = None
+            self.vte.set_clear_background(True)
+            err('error loading background image: %s, %s' % (type(e).__name__,e))
+
     def get_vte(self):
         """This simply returns the vte widget we are using"""
         return(self.vte)
@@ -387,7 +365,7 @@ class Terminal(Gtk.VBox):
                         (name, urlplugin.__class__.__name__,
                         self.matches[name]))
             except Exception as ex:
-                err('Exception occurred adding plugin URL match: %s' % ex)
+                err('Exception occurred adding plugin URL match: %s, %s' % (type(ex).__name__, ex))
 
     def match_add(self, name, match):
         """Register a URL match"""
@@ -497,7 +475,7 @@ class Terminal(Gtk.VBox):
 
         menu = self.populate_group_menu()
         menu.show_all()
-        menu.popup(None, None, self.position_popup_group_menu, widget, button, time)
+        menu.popup_at_widget(widget,Gdk.Gravity.SOUTH_WEST,Gdk.Gravity.NORTH_WEST,None)
         return(True)
 
     def populate_group_menu(self):
@@ -536,6 +514,16 @@ class Terminal(Gtk.VBox):
             item.connect('activate', self.ungroup, self.group)
             menu.append(item)
 
+        if util.has_ancestor(self, Gtk.Window):
+            item = Gtk.MenuItem.new_with_mnemonic(_('G_roup all in window'))
+            item.connect('activate', lambda x: self.emit('group_win'))
+            menu.append(item)
+
+            if len(self.terminator.groups) > 0:
+                item = Gtk.MenuItem.new_with_mnemonic(_('Ungro_up all in window'))
+                item.connect('activate', lambda x: self.emit('ungroup_win'))
+                menu.append(item)
+
         if util.has_ancestor(self, Gtk.Notebook):
             item = Gtk.MenuItem.new_with_mnemonic(_('G_roup all in tab'))
             item.connect('activate', lambda x: self.emit('group_tab'))
@@ -569,7 +557,7 @@ class Terminal(Gtk.VBox):
                           _('Broadcast _off'):'off'}.items()):
             item = Gtk.RadioMenuItem.new_with_mnemonic(groupitems, key)
             groupitems = item.get_group()
-            dbg('Terminal::populate_group_menu: %s active: %s' %
+            dbg('%s active: %s' %
                     (key, self.terminator.groupsend ==
                         self.terminator.groupsend_type[value]))
             item.set_active(self.terminator.groupsend ==
@@ -598,40 +586,22 @@ class Terminal(Gtk.VBox):
         item.connect('activate', lambda x: self.emit('enumerate', False))
         menu.append(item)
 
-        item = Gtk.MenuItem.new_with_mnemonic(_('Insert _padded terminal number'))
+        item = Gtk.MenuItem.new_with_mnemonic(_('Insert zero _padded terminal number'))
         item.connect('activate', lambda x: self.emit('enumerate', True))
         menu.append(item)
 
+        item = Gtk.MenuItem.new_with_mnemonic(_('Insert terminal _name'))
+        item.connect('activate', lambda x: self.emit('insert-term-name'))
+        menu.append(item)
+
         return(menu)
 
-    def position_popup_group_menu(self, menu, *args):
-        """Calculate the position of the group popup menu"""
-        # GTK API, or GIR just changed the args. See LP#1518058
-        widget = args[-1]
-        _screen_w = Gdk.Screen.width()
-        screen_h = Gdk.Screen.height()
-
-        widget_win = widget.get_window()
-        _something, widget_x, widget_y = widget_win.get_origin()
-        _widget_w = widget_win.get_width()
-        widget_h = widget_win.get_height()
-
-        _menu_w = menu.size_request().width
-        menu_h = menu.size_request().height
-
-        if widget_y + widget_h + menu_h > screen_h:
-            menu_y = max(widget_y - menu_h, 0)
-        else:
-            menu_y = widget_y + widget_h
-
-        return(widget_x, menu_y, 1)
-
     def set_group(self, _item, name):
         """Set a particular group"""
         if self.group == name:
             # already in this group, no action needed
             return
-        dbg('Terminal::set_group: Setting group to %s' % name)
+        dbg('Setting group to %s' % name)
         self.group = name
         self.titlebar.set_group_label(name)
         self.terminator.group_hoover()
@@ -658,7 +628,7 @@ class Terminal(Gtk.VBox):
         """Set the groupsend mode"""
         # FIXME: Can we think of a smarter way of doing this than poking?
         if value in list(self.terminator.groupsend_type.values()):
-            dbg('Terminal::set_groupsend: setting groupsend to %s' % value)
+            dbg('setting groupsend to %s' % value)
             self.terminator.groupsend = value
 
     def do_splittogroup_toggle(self):
@@ -685,8 +655,6 @@ class Terminal(Gtk.VBox):
             self.cnxids.new(self.vte, 'child-exited',
                                             lambda x, y: self.emit('close-term'))
 
-        if self.custom_encoding != True:
-            self.vte.set_encoding(self.config['encoding'])
         # Word char support was missing from vte 0.38, silently skip this setting
         if hasattr(self.vte, 'set_word_char_exceptions'):
             self.vte.set_word_char_exceptions(self.config['word_chars'])
@@ -747,7 +715,9 @@ class Terminal(Gtk.VBox):
                 pass
         self.vte.set_allow_bold(self.config['allow_bold'])
         if hasattr(self.vte,'set_cell_height_scale'): 
-            self.vte.set_cell_height_scale(self.config['line_height'])
+            self.vte.set_cell_height_scale(self.config['cell_height'])
+        if hasattr(self.vte,'set_cell_width_scale'):
+            self.vte.set_cell_width_scale(self.config['cell_width'])
         if hasattr(self.vte, 'set_bold_is_bright'):
             self.vte.set_bold_is_bright(self.config['bold_is_bright'])
 
@@ -760,28 +730,84 @@ class Terminal(Gtk.VBox):
             self.bgcolor = Gdk.RGBA()
             self.bgcolor.parse(self.config['background_color'])
 
-        if self.config['background_type'] == 'transparent' or self.config['background_type'] == 'image':
+        if self.config['background_type'] in ('transparent', 'image'):
             self.bgcolor.alpha = self.config['background_darkness']
         else:
             self.bgcolor.alpha = 1
 
+        if self.config['background_type'] == 'image' and self.config['background_image'] != '':
+            self.set_background_image(self.config['background_image'])
+        else:
+            self.background_image = None
+
+        factor = self.config['inactive_color_offset']
+        if factor > 1.0:
+          factor = 1.0
+        self.fgcolor_inactive = self.fgcolor_active.copy()
+        dbg(("fgcolor_inactive set to: RGB(%s,%s,%s)", getattr(self.fgcolor_inactive, "red"),
+                                                      getattr(self.fgcolor_inactive, "green"),
+                                                      getattr(self.fgcolor_inactive, "blue")))
+
+        for bit in ['red', 'green', 'blue']:
+            setattr(self.fgcolor_inactive, bit,
+                    getattr(self.fgcolor_inactive, bit) * factor)
+
+        dbg(("fgcolor_inactive set to: RGB(%s,%s,%s)", getattr(self.fgcolor_inactive, "red"),
+                                                      getattr(self.fgcolor_inactive, "green"),
+                                                      getattr(self.fgcolor_inactive, "blue")))
+
+        bg_factor = self.config['inactive_bg_color_offset']
+        if bg_factor > 1.0:
+            bg_factor = 1.0
+        self.bgcolor_inactive = self.bgcolor.copy()
+        dbg(("bgcolor_inactive set to: RGB(%s,%s,%s)", getattr(self.bgcolor_inactive, "red"),
+                                                      getattr(self.bgcolor_inactive, "green"),
+                                                      getattr(self.bgcolor_inactive, "blue")))
+
+        for bit in ['red', 'green', 'blue']:
+            setattr(self.bgcolor_inactive, bit,
+                    getattr(self.bgcolor_inactive, bit) * bg_factor)
+        dbg(("bgcolor_inactive set to: RGB(%s,%s,%s)", getattr(self.bgcolor_inactive, "red"),
+                                                      getattr(self.bgcolor_inactive, "green"),
+                                                      getattr(self.bgcolor_inactive, "blue")))
+
         colors = self.config['palette'].split(':')
         self.palette_active = []
-
         for color in colors:
             if color:
                 newcolor = Gdk.RGBA()
                 newcolor.parse(color)
                 self.palette_active.append(newcolor)
-        self.vte.set_colors(self.fgcolor_active, self.bgcolor,
-                                self.palette_active)
-
+        if len(colors) == 16:
+            # RGB values for indices 16..255 copied from vte source in order to dim them
+            shades = [0, 95, 135, 175, 215, 255]
+            for r in range(0, 6):
+                for g in range(0, 6):
+                    for b in range(0, 6):
+                        newcolor = Gdk.RGBA()
+                        setattr(newcolor, "red",   shades[r] / 255.0)
+                        setattr(newcolor, "green", shades[g] / 255.0)
+                        setattr(newcolor, "blue",  shades[b] / 255.0)
+                        self.palette_active.append(newcolor)
+            for y in range(8, 248, 10):
+                newcolor = Gdk.RGBA()
+                setattr(newcolor, "red",   y / 255.0)
+                setattr(newcolor, "green", y / 255.0)
+                setattr(newcolor, "blue",  y / 255.0)
+                self.palette_active.append(newcolor)
+        self.palette_inactive = []
+        for color in self.palette_active:
+            newcolor = Gdk.RGBA()
+            for bit in ['red', 'green', 'blue']:
+                setattr(newcolor, bit,
+                        getattr(color, bit) * factor)
+            self.palette_inactive.append(newcolor)
         if self.terminator.last_focused_term == self:
-            self.vte.dim(False)
-            self.queue_draw()
+            self.vte.set_colors(self.fgcolor_active, self.bgcolor,
+                                self.palette_active)
         else:
-            self.vte.dim(True)
-            self.queue_draw()
+            self.vte.set_colors(self.fgcolor_inactive, self.bgcolor_inactive,
+                                self.palette_inactive)
         profiles = self.config.base.profiles
         terminal_box_style_context = self.terminalbox.get_style_context()
         for profile in list(profiles.keys()):
@@ -837,12 +863,24 @@ class Terminal(Gtk.VBox):
 
     def set_cursor_color(self):
         """Set the cursor color appropriately"""
-        if self.config['cursor_color_fg']:
+        if self.config['cursor_color_default']:
             self.vte.set_color_cursor(None)
+            self.vte.set_color_cursor_foreground(None)
         else:
-            cursor_color = Gdk.RGBA()
-            cursor_color.parse(self.config['cursor_color'])
-            self.vte.set_color_cursor(cursor_color)
+            # foreground
+            cursor_fg_color = Gdk.RGBA()
+            if self.config['cursor_fg_color'] == '':
+                cursor_fg_color.parse(self.config['background_color'])
+            else:
+                cursor_fg_color.parse(self.config['cursor_fg_color'])
+            self.vte.set_color_cursor_foreground(cursor_fg_color)
+            # background
+            cursor_bg_color = Gdk.RGBA()
+            if self.config['cursor_bg_color'] == '':
+                cursor_bg_color.parse(self.config['foreground_color'])
+            else:
+                cursor_bg_color.parse(self.config['cursor_bg_color'])
+            self.vte.set_color_cursor(cursor_bg_color)
 
     def get_window_title(self):
         """Return the window title"""
@@ -866,7 +904,13 @@ class Terminal(Gtk.VBox):
                 focused=self.get_toplevel().get_focussed_terminal()
                 if focused in targets: targets.remove(focused)
                 if self != focused:
-                    if self.group == focused.group:
+                    if focused.group is None and self.group is None:
+                        # Create a new group and assign currently focused
+                        # terminal to this group
+                        new_group = self.terminator.new_random_group()
+                        focused.set_group(None, new_group)
+                        focused.titlebar.update()
+                    elif self.group == focused.group:
                         new_group = None
                     else:
                         new_group = focused.group
@@ -892,32 +936,18 @@ class Terminal(Gtk.VBox):
     def on_keypress(self, widget, event):
         """Handler for keyboard events"""
         if not event:
-            dbg('Terminal::on_keypress: Called on %s with no event' % widget)
+            dbg('Called on %s with no event' % widget)
             return False
 
         # FIXME: Does keybindings really want to live in Terminator()?
         mapping = self.terminator.keybindings.lookup(event)
 
-        # Just propagate tab-swictch events if there is only one tab
-        if (
-                mapping and (
-                    mapping.startswith('switch_to_tab') or
-                    mapping in ('next_tab', 'prev_tab')
-                )
-        ):
-            window = self.get_toplevel()
-            child = window.get_children()[0]
-            if isinstance(child, Terminal):
-                # not a Notebook instance => a single tab is used
-                # .get_n_pages() can not be used
-                return False
-
         if mapping == "hide_window":
             return False
 
         if mapping and mapping not in ['close_window',
                                        'full_screen']:
-            dbg('Terminal::on_keypress: lookup found: %r' % mapping)
+            dbg('lookup found: %r' % mapping)
             # handle the case where user has re-bound copy to ctrl+<key>
             # we only copy if there is a selection otherwise let it fall through
             # to ^<key>
@@ -956,15 +986,15 @@ class Terminal(Gtk.VBox):
 
         if self.config['putty_paste_style']:
             middle_click = [self.popup_menu, (widget, event)]
-            right_click = [self.paste_clipboard, (not self.config['putty_paste_style_source_clipboard'], )]
+            right_click = [self.paste_clipboard, (not self.config['putty_paste_style_source_clipboard'], True)]
         else:
-            middle_click = [self.paste_clipboard, (True, )]
+            middle_click = [self.paste_clipboard, (True, True)]
             right_click = [self.popup_menu, (widget, event)]
 
         # Ctrl-click event here.
         if event.button == self.MOUSEBUTTON_LEFT:
             # Ctrl+leftclick on a URL should open it
-            if event.get_state() & Gdk.ModifierType.CONTROL_MASK == Gdk.ModifierType.CONTROL_MASK:
+            if self.config["link_single_click"] or event.get_state() & Gdk.ModifierType.CONTROL_MASK == Gdk.ModifierType.CONTROL_MASK:
                 # Check new OSC-8 method first
                 url = self.vte.hyperlink_check_event(event)
                 dbg('url: %s' % url)
@@ -975,6 +1005,8 @@ class Terminal(Gtk.VBox):
                     url = self.vte.match_check_event(event)
                     if url[0]:
                         self.open_url(url, prepare=True)
+                    else:
+                        dbg("No regex match, discard event.")
         elif event.button == self.MOUSEBUTTON_MIDDLE:
             # middleclick should paste the clipboard
             # try to pass it to vte widget first though
@@ -1006,7 +1038,9 @@ class Terminal(Gtk.VBox):
         """Handler for modifier + mouse wheel scroll events"""
         SMOOTH_SCROLL_UP = event.direction == Gdk.ScrollDirection.SMOOTH and event.delta_y <= 0.
         SMOOTH_SCROLL_DOWN = event.direction == Gdk.ScrollDirection.SMOOTH and event.delta_y > 0.
-        if event.state & Gdk.ModifierType.CONTROL_MASK == Gdk.ModifierType.CONTROL_MASK:
+
+        modifiers = event.state & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        if modifiers == Gdk.ModifierType.CONTROL_MASK:
             # Zoom the terminal(s) in or out if not disabled in config
             if self.config["disable_mousewheel_zoom"] is True:
                 return False
@@ -1025,7 +1059,7 @@ class Terminal(Gtk.VBox):
                 for target in targets:
                     target.zoom_out()
                 return True
-        if event.state & Gdk.ModifierType.SHIFT_MASK == Gdk.ModifierType.SHIFT_MASK:
+        elif modifiers == Gdk.ModifierType.SHIFT_MASK:
             # Shift + mouse wheel up/down
             if event.direction == Gdk.ScrollDirection.UP or SMOOTH_SCROLL_UP:
                 self.scroll_by_page(-1)
@@ -1042,6 +1076,9 @@ class Terminal(Gtk.VBox):
         menu = TerminalPopupMenu(self)
         menu.show(widget, event)
 
+    def do_readonly_toggle(self):
+        self.vte.props.input_enabled = not self.vte.props.input_enabled
+
     def do_scrollbar_toggle(self):
         """Show or hide the terminal scrollbar"""
         self.toggle_widget_visibility(self.scrollbar)
@@ -1053,14 +1090,6 @@ class Terminal(Gtk.VBox):
         else:
             widget.show()
 
-    def on_encoding_change(self, _widget, encoding):
-        """Handle the encoding changing"""
-        current = self.vte.get_encoding()
-        if current != encoding:
-            dbg('on_encoding_change: setting encoding to: %s' % encoding)
-            self.custom_encoding = not (encoding == self.config['encoding'])
-            self.vte.set_encoding(encoding)
-
     def on_drag_begin(self, widget, drag_context, _data):
         """Handle the start of a drag event"""
         Gtk.drag_set_icon_pixbuf(drag_context, util.widget_pixbuf(self, 512), 0, 0)
@@ -1119,23 +1148,63 @@ class Terminal(Gtk.VBox):
         connec = widget.connect_after('draw', self.on_draw)
         widget.queue_draw_area(0, 0, alloc.width, alloc.height)
         widget.get_window().process_updates(True)
-        # finaly reset the values
+        # finally reset the values
         widget.disconnect(connec)
         widget._draw_data = None
 
     def background_draw(self, widget, cr):
-        if not self.config['background_type'] == 'image' or not self.background_image:
+        if self.background_image is None:
             return False
 
+        # save cairo context
+        cr.save()
+
+        # draw background image
+        image_mode = self.config['background_image_mode']
+        image_align_horiz = self.config['background_image_align_horiz']
+        image_align_vert = self.config['background_image_align_vert']
+
         rect = self.vte.get_allocation()
         xratio = float(rect.width) / float(self.background_image.get_width())
         yratio = float(rect.height) / float(self.background_image.get_height())
-        cr.save()
-        cr.scale(xratio,yratio)
-        Gdk.cairo_set_source_pixbuf(cr, self.background_image, 0, 0)
+        if image_mode == 'stretch_and_fill':
+            # keep stretched ratios
+            xratio = xratio
+            yratio = yratio
+        elif image_mode == 'scale_and_fit':
+            ratio = min(xratio, yratio)
+            xratio = yratio = ratio
+        elif image_mode == 'scale_and_crop':
+            ratio = max(xratio, yratio)
+            xratio = yratio = ratio
+        else:
+            xratio = yratio = 1
+        cr.scale(xratio, yratio)
+
+        xoffset = 0
+        yoffset = 0
+        if image_align_horiz == 'center':
+            xoffset = (rect.width / xratio - self.background_image.get_width()) / 2
+        elif image_align_horiz == 'right':
+            xoffset = rect.width / xratio - self.background_image.get_width()
+
+        if image_align_vert == 'middle':
+            yoffset = (rect.height / yratio - self.background_image.get_height()) / 2
+        elif image_align_vert == 'bottom':
+            yoffset = rect.height / yratio - self.background_image.get_height()
+
+        cr.set_source_surface(self.background_image, xoffset, yoffset)
+        cr.get_source().set_filter(cairo.Filter.FAST)
+        if image_mode == 'tiling':
+            cr.get_source().set_extend(cairo.Extend.REPEAT)
+
         cr.paint()
-        Gdk.cairo_set_source_rgba(cr,self.bgcolor)
+
+        # draw transparent monochrome layer
+        Gdk.cairo_set_source_rgba(cr, self.bgcolor)
         cr.paint()
+
+        # restore cairo context
         cr.restore()
 
     def on_draw(self, widget, context):
@@ -1231,7 +1300,7 @@ class Terminal(Gtk.VBox):
     def get_location(self, term, x, y):
         """Get our location within the terminal"""
         pos = ''
-        # get the diagonales function for the receiving widget
+        # get the diagonals function for the receiving widget
         term_alloc = term.get_allocation()
         coef1 = float(term_alloc.height)/float(term_alloc.width)
         coef2 = -float(term_alloc.height)/float(term_alloc.width)
@@ -1258,11 +1327,11 @@ class Terminal(Gtk.VBox):
 
     def grab_focus(self):
         """Steal focus for this terminal"""
-        if not self.vte.has_focus():
+        if self.vte and not self.vte.has_focus():
             self.vte.grab_focus()
 
     def ensure_visible_and_focussed(self):
-        """Make sure that we're visible and focussed"""
+        """Make sure that we're visible and focused"""
         window = self.get_toplevel()
         try:
             topchild = window.get_children()[0]
@@ -1287,8 +1356,9 @@ class Terminal(Gtk.VBox):
 
     def on_vte_focus_in(self, _widget, _event):
         """Inform other parts of the application when focus is received"""
-        self.vte.dim(False)
-        self.queue_draw()
+        self.vte.set_colors(self.fgcolor_active, self.bgcolor,
+                            self.palette_active)
+        self.set_cursor_color()
         if not self.terminator.doing_layout:
             self.terminator.last_focused_term = self
             if self.get_toplevel().is_child_notebook():
@@ -1302,8 +1372,9 @@ class Terminal(Gtk.VBox):
 
     def on_vte_focus_out(self, _widget, _event):
         """Inform other parts of the application when focus is lost"""
-        self.vte.dim(True)
-        self.queue_draw()
+        self.vte.set_colors(self.fgcolor_inactive, self.bgcolor_inactive,
+                            self.palette_inactive)
+        self.set_cursor_color()
         self.emit('focus-out')
 
     def on_window_focus_out(self):
@@ -1342,6 +1413,9 @@ class Terminal(Gtk.VBox):
         if self.config['geometry_hinting']:
             window = self.get_toplevel()
             window.deferred_set_rough_geometry_hints()
+        else:
+            window = self.get_toplevel()
+            window.disable_geometry_hints()
 
     def on_vte_notify_enter(self, term, event):
         """Handle the mouse entering this terminal"""
@@ -1370,14 +1444,14 @@ class Terminal(Gtk.VBox):
     def zoom_scale(self, widget, allocation, old_data):
         """Scale our font correctly based on how big we are not vs before"""
         self.cnxids.remove_signal(self, 'size-allocate')
-        # FIXME: Is a zoom signal actualy used anywhere?
+        # FIXME: Is a zoom signal actually used anywhere?
         self.cnxids.remove_signal(self, 'zoom')
 
         new_columns = self.vte.get_column_count()
         new_rows = self.vte.get_row_count()
         new_font = self.vte.get_font()
 
-        dbg('Terminal::zoom_scale: Resized from %dx%d to %dx%d' % (
+        dbg('Resized from %dx%d to %dx%d' % (
              old_data['old_columns'],
              old_data['old_rows'],
              new_columns,
@@ -1385,7 +1459,7 @@ class Terminal(Gtk.VBox):
 
         if new_rows == old_data['old_rows'] or \
            new_columns == old_data['old_columns']:
-            dbg('Terminal::zoom_scale: One axis unchanged, not scaling')
+            dbg('One axis unchanged, not scaling')
             return
 
         scale_factor = min ( (new_columns / old_data['old_columns'] * 0.97),
@@ -1401,15 +1475,8 @@ class Terminal(Gtk.VBox):
 
     def is_zoomed(self):
         """Determine if we are a zoomed terminal"""
-        prop = None
         window = self.get_toplevel()
-
-        try:
-            prop = window.get_property('term-zoomed')
-        except TypeError:
-            prop = False
-
-        return prop
+        return window.is_zoomed()
 
     def zoom(self, widget=None):
         """Zoom ourself to fill the window"""
@@ -1432,10 +1499,10 @@ class Terminal(Gtk.VBox):
         self.is_held_open = True
         self.titlebar.update()
 
-    def spawn_child(self, widget=None, respawn=False, debugserver=False):
+    def spawn_child(self, widget=None, respawn=False, debugserver=False, init_command=None):
         args = []
         shell = None
-        command = None
+        command = init_command
 
         if self.terminator.doing_layout:
             dbg('still laying out, refusing to spawn a child')
@@ -1463,9 +1530,10 @@ class Terminal(Gtk.VBox):
             command = self.layout_command
         elif debugserver is True:
             details = self.terminator.debug_address
-            dbg('spawning debug session with: %s:%s' % (details[0],
-                details[1]))
-            command = 'telnet %s %s' % (details[0], details[1])
+            if details is not None:
+                dbg('spawning debug session with: %s:%s' % (details[0],
+                    details[1]))
+                command = 'telnet %s %s' % (details[0], details[1])
 
         # working directory set in layout config
         if self.directory:
@@ -1509,14 +1577,37 @@ class Terminal(Gtk.VBox):
 
         dbg('Forking shell: "%s" with args: %s' % (shell, args))
         args.insert(0, shell)
-        result,  self.pid = self.vte.spawn_sync(Vte.PtyFlags.DEFAULT,
-                                                self.cwd,
-                                                args,
-                                                envv,
-                                                GLib.SpawnFlags.FILE_AND_ARGV_ZERO,
-                                                None,
-                                                None,
-                                                None)
+
+        if util.is_flatpak():
+            dbg('Flatpak detected')
+            args = util.get_flatpak_args(args, envv, self.cwd)
+            dbg('Forking shell: "%s" with args: %s via flatpak-spawn' % (shell, args))
+        
+            self.pid = self.vte.spawn_async(
+                Vte.PtyFlags.NO_CTTY,
+                self.cwd,
+                args,
+                envv,
+                0,
+                None,
+                None,
+                -1,
+                None,
+                None,
+                None,
+            )
+        else:
+            result, self.pid = self.vte.spawn_sync(
+                    Vte.PtyFlags.DEFAULT,
+                    self.cwd,
+                    args,
+                    envv,
+                    GLib.SpawnFlags.FILE_AND_ARGV_ZERO,
+                    None,
+                    None,
+                    None
+                    )
+
         self.command = shell
 
         self.titlebar.update()
@@ -1540,17 +1631,18 @@ class Terminal(Gtk.VBox):
                 registry = plugin.PluginRegistry()
                 registry.load_plugins()
                 plugins = registry.get_plugins_by_capability('url_handler')
+                dbg("URL handler plugins: {}".format(plugins))
 
                 for urlplugin in plugins:
                     if match == self.matches[urlplugin.handler_name]:
                         newurl = urlplugin.callback(url)
-                        if newurl is not None:
-                            dbg('Terminal::prepare_url: URL prepared by \
-%s plugin' % urlplugin.handler_name)
+                        if newurl: # If the plugin returns None, it's a false match.
+                            dbg('URL prepared by %s plugin' \
+                                    % urlplugin.handler_name)
                             url = newurl
                         break
             except Exception as ex:
-                err('Exception occurred preparing URL: %s' % ex)
+                err('Exception occurred preparing URL: %s, %s' % (type(ex).__name__, ex))
 
         return url
 
@@ -1558,8 +1650,15 @@ class Terminal(Gtk.VBox):
         """Open a given URL, conditionally unpacking it from a VTE match"""
         if prepare:
             url = self.prepare_url(url)
-        dbg('open_url: URL: %s (prepared: %s)' % (url, prepare))
+        dbg('URL: %s (prepared: %s)' % (url, prepare))
 
+        # If the URL opening is managed by the plugin: do nothing.
+        # (plugins can indicate they manage the URL opening by returning a "terminator://" URI).
+        if url.split(":")[0] == "terminator":
+            dbg("URL opening is managed by the plugin, do nothing more.")
+            return
+
+        # Else, call the URL handler.
         if self.config['use_custom_url_handler']:
             dbg("Using custom URL handler: %s" %
                 self.config['custom_url_handler'])
@@ -1582,18 +1681,20 @@ class Terminal(Gtk.VBox):
             import webbrowser
             webbrowser.open(url)
 
-    def paste_clipboard(self, primary=False):
+
+    def paste_clipboard(self, primary=False, mouse=False):
         """Paste one of the two clipboards"""
-        for term in self.terminator.get_target_terms(self):
-            if primary:
-                term.vte.paste_primary()
-            else:
-                term.vte.paste_clipboard()
-        self.vte.grab_focus()
+        if not (mouse and self.config['disable_mouse_paste']):
+            for term in self.terminator.get_target_terms(self):
+                if primary:
+                    term.vte.paste_primary()
+                else:
+                    term.vte.paste_clipboard()
+            self.vte.grab_focus()
 
     def feed(self, text):
         """Feed the supplied text to VTE"""
-        self.vte.feed_child(text)
+        self.vte.feed_child(text.encode())
 
     def zoom_in(self):
         """Increase the font size"""
@@ -1623,7 +1724,7 @@ class Terminal(Gtk.VBox):
             font = self.config.get_system_mono_font()
         else:
             font = self.config['font']
-        dbg("Terminal::zoom_orig: restoring font to: %s" % font)
+        dbg("restoring font to: %s" % font)
         self.set_font(Pango.FontDescription(font))
         self.custom_font_size = None
 
@@ -1632,7 +1733,7 @@ class Terminal(Gtk.VBox):
         self.vte.set_font(fontdesc)
 
     def get_cursor_position(self):
-        """Return the co-ordinates of our cursor"""
+        """Return the coordinates of our cursor"""
         # FIXME: THIS METHOD IS DEPRECATED AND UNUSED
         col, row = self.vte.get_cursor_position()
         width = self.vte.get_char_width()
@@ -1673,7 +1774,7 @@ class Terminal(Gtk.VBox):
             connec = widget.connect_after('draw', self.on_draw)
             widget.queue_draw_area(0, 0, alloc.width, alloc.height)
             widget.get_window().process_updates(True)
-            # finaly reset the values
+            # finally reset the values
             widget.disconnect(connec)
             widget._draw_data = None
 
@@ -1686,7 +1787,7 @@ class Terminal(Gtk.VBox):
         widget.get_window().process_updates(True)
         return False
 
-    def describe_layout(self, count, parent, global_layout, child_order):
+    def describe_layout(self, count, parent, global_layout, child_order, save_cwd = False):
         """Describe our layout"""
         layout = {'type': 'Terminal', 'parent': parent, 'order': child_order}
         if self.group:
@@ -1699,6 +1800,8 @@ class Terminal(Gtk.VBox):
         if title:
             layout['title'] = title
         layout['uuid'] = self.uuid
+        if save_cwd:
+            layout['directory'] = self.get_cwd()
         name = 'terminal%d' % count
         count = count + 1
         global_layout[name] = layout
@@ -1770,6 +1873,9 @@ class Terminal(Gtk.VBox):
     def key_paste(self):
         self.paste_clipboard()
 
+    def key_paste_selection(self):
+        self.paste_clipboard(True)
+
     def key_toggle_scrollbar(self):
         self.do_scrollbar_toggle()
 
@@ -1816,6 +1922,9 @@ class Terminal(Gtk.VBox):
     def key_go_right(self):
         self.emit('navigate', 'right')
 
+    def key_split_auto(self):
+        self.emit('split-auto', self.get_cwd())
+
     def key_split_horiz(self):
         self.emit('split-horiz', self.get_cwd())
 
@@ -1915,6 +2024,16 @@ class Terminal(Gtk.VBox):
     def key_ungroup_all(self):
         self.emit('ungroup-all')
 
+    def key_group_win(self):
+        dbg("Group Win")
+        self.emit('group-win')
+
+    def key_group_win_toggle(self):
+        self.emit('group-win-toggle')
+
+    def key_ungroup_win(self):
+        self.emit('ungroup-win')
+
     def key_group_tab(self):
         self.emit('group-tab')
 
@@ -2019,6 +2138,11 @@ class Terminal(Gtk.VBox):
     def key_preferences(self):
         PrefsEditor(self)
 
+    def key_preferences_keybindings(self):
+        #need to have this as a config may be preferences_default
+        #have a mapping rather than hardcoded page
+        PrefsEditor(self, cur_page = 3)
+
     def key_help(self):
         manual_index_page = manual_lookup()
         if manual_index_page:
